/*******************************************************************************
 *
 * File Name: ANNdisplay.cpp
 * Author: Richard Baxter
 * Project: Advanced Neural Network (ANN)
 * Project Version: 2a4d 24-Oct-09
 * Comments: TH = Test Harness
 *
 *******************************************************************************/

#include "LDparser.h"
#include "ANNdisplay.h"
#include "ANNFormation.h"
#include "ANNTraining.h"
#include "ANNXMLconversion.h"
#include "ANNsprite.h"
#include "ANNUpdateAlgorithm.h"
#include "LDreferenceManipulation.h"

#ifdef USE_RT
#include "RTscene.h"
#include "RTreferenceManipulation.h"
#include "RTpixelMaps.h"
#endif


#ifdef USE_RT

void generateExperienceWith2DRGBMap(unsigned char * rgbMap, int imageWidth, int imageHeight, Experience * currentExperience, long objectDecision)
{
	currentExperience->classTargetValue = objectDecision;
	ExperienceInput * currentExperienceInput = currentExperience->firstExperienceInput;
	
	//fill experience inputs
	for(int y=0; y<imageHeight; y++)
	{
		for(int x=0; x<imageWidth; x++)
		{	
			//cout << "x = " << x << endl;
			//cout << "y = " << y << endl;
			
			for(int rgb=0; rgb<RGB_NUM; rgb++)
			{
				unsigned char col = getRGBMapValue(x, y, imageWidth, rgb, rgbMap);
				double normalisedMapValue =  normaliseExperienceInput((double)col, MAX_RGB_VAL);
				currentExperienceInput->inputValue = normalisedMapValue;
				ExperienceInput * newExperienceInput = new ExperienceInput();
				currentExperienceInput->next = newExperienceInput;
				currentExperienceInput = currentExperienceInput->next;			
			}
		}	
	}
}


void generateExperienceWith2DMap(double * lumOrContrastOrDepthMap, int imageWidth, int imageHeight, double mapMaxValue, Experience * currentExperience, long objectDecision)
{
	currentExperience->classTargetValue = objectDecision;
	ExperienceInput * currentExperienceInput = currentExperience->firstExperienceInput;
	
	//fill experience inputs
	for(int y=0; y<imageHeight; y++)
	{
		for(int x=0; x<imageWidth; x++)
		{	
			//cout << "x = " << x << endl;
			//cout << "y = " << y << endl;
			
			double mapValue = getLumOrContrastOrDepthMapValue(x, y, imageWidth,  lumOrContrastOrDepthMap);

			double normalisedMapValue =  normaliseExperienceInput(mapValue, mapMaxValue);
			currentExperienceInput->inputValue = normalisedMapValue;
			ExperienceInput * newExperienceInput = new ExperienceInput();
			currentExperienceInput->next = newExperienceInput;
			currentExperienceInput = currentExperienceInput->next;
		}	
	}
}

void generateExperienceWith2DBooleanMap(bool * booleanMap, int imageWidth, int imageHeight, Experience * currentExperience, long objectDecision)
{
	currentExperience->classTargetValue = objectDecision;
	ExperienceInput * currentExperienceInput = currentExperience->firstExperienceInput;
	
	//fill experience inputs
	for(int y=0; y<imageHeight; y++)
	{
		for(int x=0; x<imageWidth; x++)
		{	
			//cout << "x = " << x << endl;
			//cout << "y = " << y << endl;
			
			bool mapValue = getBooleanMapValue(x, y, imageWidth,  booleanMap);

			double normalisedMapValue =  normaliseExperienceInput((double)mapValue, 1.0);
			currentExperienceInput->inputValue = normalisedMapValue;
			ExperienceInput * newExperienceInput = new ExperienceInput();
			currentExperienceInput->next = newExperienceInput;
			currentExperienceInput = currentExperienceInput->next;
		}	
	}
}

#endif
	



bool trainAndOutputNeuralNetworkWithFileNames(NeuronContainer * firstInputNeuronInNetwork, NeuronContainer * firstOutputNeuronInNetwork, int numberOfInputNeurons, int numberOfOutputNeurons, Experience * firstExperienceInList, bool addSprites, bool allowRaytrace, string * XMLNNSceneFileName, char * charstarvectorGraphicsLDRNNSceneFileName, char * charstarvectorGraphicsLDRNNSceneFileNameWithSprites, char * charstarvectorGraphicsTALNNSceneFileName, char * charstarraytracedImagePPMNNSceneFileName, char * charstarexperienceNNSceneFileName, bool useFoldsDuringTraining, int maxOrSetNumEpochs)
{
	bool result = true;
	
	cout << "num experiences in list = " << countNumberOfExperiences(firstExperienceInList) << endl;

	#ifndef TH_LRRC_LOW_RAM_AVAILABLE
	long numberOfExperiences = countNumberOfExperiences(firstExperienceInList);


	if(useFoldsDuringTraining)
	{
		int maxFolds = MAX_NUM_FOLDS_ANNTH;
		int maxNumEpochs = maxOrSetNumEpochs; 
		trainNeuralNetwork(firstInputNeuronInNetwork, firstOutputNeuronInNetwork, numberOfInputNeurons, numberOfOutputNeurons, maxFolds, firstExperienceInList, numberOfExperiences, maxNumEpochs);
			//this is done dynamically if do not have heaps of RAM
	}
	else
	{
		int setNumEpochs = maxOrSetNumEpochs; //ANN_DEFAULT_SIMPLE_TRAIN_DEFAULT_NUM_OF_TRAINING_EPOCHS;
		trainNeuralNetworkSimple(firstInputNeuronInNetwork, firstOutputNeuronInNetwork, numberOfInputNeurons, numberOfOutputNeurons, setNumEpochs, firstExperienceInList, numberOfExperiences);
	}

	#endif
	
#ifndef USE_OR	
	if(!writeNetXMLFile(*XMLNNSceneFileName, firstInputNeuronInNetwork))
	{
		result = false;
	}

	outputNeuralNetworkToVectorGraphicsAndRaytrace(firstInputNeuronInNetwork, addSprites, allowRaytrace, charstarvectorGraphicsLDRNNSceneFileName, charstarvectorGraphicsLDRNNSceneFileNameWithSprites, charstarvectorGraphicsTALNNSceneFileName, charstarraytracedImagePPMNNSceneFileName);


		
	//cout << "H6" << endl;

	#ifndef TH_LRRC_LOW_RAM_AVAILABLE

	writeExperienceListToFile(charstarexperienceNNSceneFileName, firstExperienceInList);
	
	#endif

#endif

	return result;
}

void outputNeuralNetworkToVectorGraphicsAndRaytrace(NeuronContainer * firstInputNeuronInNetwork, bool addSprites, bool allowRaytrace, char * charstarvectorGraphicsLDRNNSceneFileName, char * charstarvectorGraphicsLDRNNSceneFileNameWithSprites, char * charstarvectorGraphicsTALNNSceneFileName, char * charstarraytracedImagePPMNNSceneFileName)
{
	bool result = true;
	
	//now output the network to vector graphics file
	if(allowRaytrace)
	{
		//now output the vector graphics file to image file via ray tracer

	#ifdef TH_USE_RT_FOR_NEURAL_NETWORK_VEC_GRAPHICS
		ANNcreateNeuralNetworkSceneFilesWithAndWithoutSprites(charstarvectorGraphicsLDRNNSceneFileName, charstarvectorGraphicsLDRNNSceneFileNameWithSprites, firstInputNeuronInNetwork, addSprites);

		char * charstarsceneFileNameForRayTracing;
		
		
		if(addSprites)
		{
			charstarsceneFileNameForRayTracing = charstarvectorGraphicsLDRNNSceneFileNameWithSprites;
		}
		else
		{
			charstarsceneFileNameForRayTracing = charstarvectorGraphicsLDRNNSceneFileName;
		}
		
		// OLD; if using RT, do not ray trace sprites as the RT raytracer is not optimised - use povray instead
		//charstarsceneFileNameForRayTracing = charstarvectorGraphicsLDRNNSceneFileName;
		// NEW; use ANNrules.xml to remove sprites for RT speed
		
		//reparse scenefilewithandwithout sprites - to build absolute position information
		Reference * initialReferenceInSceneFileForRayTracing = new Reference();
		Reference * topLevelReferenceInSceneFileForRayTracing = new Reference(charstarsceneFileNameForRayTracing, 1, true);	//The information in this object is not required or meaningful, but needs to be passed into the parseFile/parseReferenceList recursive function
		if(!parseFile(charstarsceneFileNameForRayTracing, initialReferenceInSceneFileForRayTracing, topLevelReferenceInSceneFileForRayTracing, true))
		{//file does not exist
			cout << "The file: " << charstarsceneFileNameForRayTracing << " does not exist in the directory" << endl;
			result = false;
		}

		if(!write2DReferenceListCollapsedTo1DToFileRayTraceFormat(charstarvectorGraphicsTALNNSceneFileName, initialReferenceInSceneFileForRayTracing, false, NULL, false, NULL, NULL))
		{
			result = false;
		}

		if(!rayTraceScene(charstarvectorGraphicsTALNNSceneFileName, charstarraytracedImagePPMNNSceneFileName, true, false, NULL, NULL, NULL, NULL))
		{
			result = false;
		}		
	#else
		#ifdef TH_USE_POVRAY_FOR_NEURAL_NETWORK_VEC_GRAPHICS	//NOT YET TEST;
			
		
			Reference * nonspriteListInitialReference = new Reference();
			Reference * spriteListInitialReference = new Reference();

			int numSpritesAdded = 0;

			if(!ANNcreateNeuralNetworkReferenceListsWithAndWithoutSprites(charstarvectorGraphicsLDRNNSceneFileNameWithSprites, nonspriteListInitialReference, spriteListInitialReference, firstInputNeuronInNetwork, addSprites, &numSpritesAdded))
			{
				result = false;
			}

			if(!ANNcreateNeuralNetworkSceneFilesWithAndWithoutSpritesFromReferenceLists(charstarvectorGraphicsLDRNNSceneFileName, charstarvectorGraphicsLDRNNSceneFileNameWithSprites, addSprites, nonspriteListInitialReference, spriteListInitialReference, numSpritesAdded))
			{
				result = false;
			}

			char * charstarsceneFileNameForRayTracing; 
			if(addSprites)
			{
				if(!joinReferenceLists(nonspriteListInitialReference, spriteListInitialReference))
				{
					result = false;
				}

				charstarsceneFileNameForRayTracing = charstarvectorGraphicsLDRNNSceneFileNameWithSprites;
			}
			else
			{
				charstarsceneFileNameForRayTracing = charstarvectorGraphicsLDRNNSceneFileName;
			}


			//use third party raytracer, povray;

			string temp = "";
			string l3pCommand = temp + OBJECT_EXPERIENCE_GEN_L3P_EXCCMD_PART_1 + OBJECT_EXPERIENCE_GEN_L3P_EXCCMD_PART_2 + charstarsceneFileNameForRayTracing + OBJECT_EXPERIENCE_GEN_L3P_EXCCMD_PART_5 + OBJECT_EXPERIENCE_GEN_L3P_EXCCMD_PART_6 + OBJECT_EXPERIENCE_GEN_L3P_EXCCMD_PART_7;
			string povrayCommand = temp + OBJECT_EXPERIENCE_GEN_POVRAY_EXCCMD_PART_1 + OBJECT_EXPERIENCE_GEN_POVRAY_EXCCMD_PART_2 + OBJECT_EXPERIENCE_GEN_POVRAY_EXCCMD_PART_3 + OBJECT_EXPERIENCE_GEN_POVRAY_EXCCMD_PART_4 + OBJECT_EXPERIENCE_GEN_POVRAY_EXCCMD_PART_5 + OBJECT_EXPERIENCE_GEN_POVRAY_EXCCMD_PART_6 + charstarraytracedImagePPMNNSceneFileName + OBJECT_EXPERIENCE_GEN_L3P_EXCCMD_PART_9 + OBJECT_EXPERIENCE_GEN_POVRAY_EXCCMD_PART_10 + OBJECT_EXPERIENCE_GEN_POVRAY_EXCCMD_PART_11 + OBJECT_EXPERIENCE_GEN_POVRAY_EXCCMD_PART_12 + OBJECT_EXPERIENCE_GEN_POVRAY_EXCCMD_PART_13 + OBJECT_EXPERIENCE_GEN_POVRAY_EXCCMD_PART_14;

			//Eg system("env LDRAWDIR=/usr/share/local/LDRAW l3p /usr/share/local/LDRAW/MODELS/car.dat object -cg30,45 -b -f");
			//Eg system("povray object.pov");
			cout << l3pCommand <<endl;
			system(l3pCommand.c_str());
			cout << povrayCommand <<endl;
			system(povrayCommand.c_str());


			delete nonspriteListInitialReference;
			delete spriteListInitialReference;

		#else
			cout << "Error: no ray tracer enabled" << endl;
			exit(0);
		#endif
	#endif
	}	
	else
	{
		ANNcreateNeuralNetworkSceneFilesWithAndWithoutSprites(charstarvectorGraphicsLDRNNSceneFileName, charstarvectorGraphicsLDRNNSceneFileNameWithSprites, firstInputNeuronInNetwork, addSprites);
	}
}

void writeExperienceListToFile(char * fileName, Experience * firstExperienceInList)
{
	ofstream experienceDataSetOFStreamObject(fileName);

	Experience * currentExperience = firstExperienceInList;
	while(currentExperience->next != NULL)
	{
		addExperienceToOFStream(&experienceDataSetOFStreamObject, currentExperience);
		currentExperience = currentExperience->next;
	}

	experienceDataSetOFStreamObject.close();
}


