/*******************************************************************************
 *
 * File Name: RTpixelMaps.cpp
 * Author: Richard Baxter
 * Project: Raytracer Functions
 * Project Version: 2a4d 24-Oct-09
 *
 *******************************************************************************/
 
#include "RTpixelMaps.h"


#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <string.h>
#include <iostream>
#include <fstream>
#include <time.h>
#include <math.h>
using namespace std;





double calculateLuminosityLevelFromColour(colour * col)
{
	double luminosity = (col->r+col->g+col->b);
	//cout << "luminosity = " << luminosity << endl;
	
	return luminosity;
}



void setBooleanMapValue(int x, int y, int imageWidth, bool booleanVal, bool * booleanMap)
{
	booleanMap[y*imageWidth + x] = booleanVal;
}

bool getBooleanMapValue(int x, int y, int imageWidth, bool * booleanMap)
{
	return booleanMap[y*imageWidth + x];
}


void setIntMapValue(int x, int y, int imageWidth, int intVal, int * intMap)
{
	intMap[y*imageWidth + x] = intVal;
}

int getIntMapValue(int x, int y, int imageWidth, int * intMap)
{
	return intMap[y*imageWidth + x];
}




unsigned char getRGBMapValue(int x, int y, int imageWidth, int rgb, unsigned char * rgbMap)
{
	return rgbMap[y*imageWidth*RGB_NUM + x*RGB_NUM + rgb];
}

void getRGBMapValues(int x, int y, int imageWidth, unsigned char * rgbMap, colour * col)
{
	col->r = rgbMap[y*imageWidth*RGB_NUM + x*RGB_NUM + RGB_RED];
	col->g = rgbMap[y*imageWidth*RGB_NUM + x*RGB_NUM + RGB_GREEN];
	col->b = rgbMap[y*imageWidth*RGB_NUM + x*RGB_NUM + RGB_BLUE];
}


void setRGBMapValues(int x, int y, int imageWidth, colour * col, unsigned char * rgbMap)
{
	/*
	printf("\ncol->r = %d", col->r);
	printf("\ncol->g = %d", col->g);
	printf("\ncol->b = %d", col->b);
	*/
		
	rgbMap[y*imageWidth*RGB_NUM + x*RGB_NUM + RGB_RED] = col->r;
	rgbMap[y*imageWidth*RGB_NUM + x*RGB_NUM + RGB_GREEN] = col->g;
	rgbMap[y*imageWidth*RGB_NUM + x*RGB_NUM + RGB_BLUE] = col->b;
}

void setNormalMapValue(int x, int y, int imageWidth, vec * normalVal, double * normalMap)
{
	setVectorMapValue(x, y, imageWidth, normalVal, normalMap);
}

void getNormalMapValue(int x, int y, int imageWidth, double * normalMap, vec * normalVal)
{
	getVectorMapValue(x, y, imageWidth, normalMap, normalVal);
}

void setPointMapValue(int x, int y, int imageWidth, vec * pointVal, double * pointMap)
{
	setVectorMapValue(x, y, imageWidth, pointVal, pointMap);
}

void getPointMapValue(int x, int y, int imageWidth, double * pointMap, vec * pointVal)
{
	getVectorMapValue(x, y, imageWidth, pointMap, pointVal);
}


void setVectorMapValue(int x, int y, int imageWidth, vec * vectorVal, double * vectorMap)
{
	vectorMap[y*imageWidth*VEC_MAP_VEC_NUM_DIMENSIONS + x*VEC_MAP_VEC_NUM_DIMENSIONS + VEC_MAP_VEC_X] = vectorVal->x;
	vectorMap[y*imageWidth*VEC_MAP_VEC_NUM_DIMENSIONS + x*VEC_MAP_VEC_NUM_DIMENSIONS + VEC_MAP_VEC_Y] = vectorVal->y;
	vectorMap[y*imageWidth*VEC_MAP_VEC_NUM_DIMENSIONS + x*VEC_MAP_VEC_NUM_DIMENSIONS + VEC_MAP_VEC_Z] = vectorVal->z;
}
void getVectorMapValue(int x, int y, int imageWidth, double * vectorMap, vec * vectorVal)
{
	vectorVal->x = vectorMap[y*imageWidth*VEC_MAP_VEC_NUM_DIMENSIONS + x*VEC_MAP_VEC_NUM_DIMENSIONS + VEC_MAP_VEC_X];
	vectorVal->y = vectorMap[y*imageWidth*VEC_MAP_VEC_NUM_DIMENSIONS + x*VEC_MAP_VEC_NUM_DIMENSIONS + VEC_MAP_VEC_Y];
	vectorVal->z = vectorMap[y*imageWidth*VEC_MAP_VEC_NUM_DIMENSIONS + x*VEC_MAP_VEC_NUM_DIMENSIONS + VEC_MAP_VEC_Z];
}










void setLumOrContrastOrDepthMapValue(int x, int y, int imageWidth, double contrastVal, double * contrastMap)
{
	contrastMap[y*imageWidth + x] = contrastVal;
}

double getLumOrContrastOrDepthMapValue(int x, int y, int imageWidth, double * contrastMap)
{
	return contrastMap[y*imageWidth + x];
}




void generatePixmapFromRGBMap(char * imageFileName, int imageWidth, int imageHeight, unsigned char * rgbMap)
{
	int x,y;
	pixmap* pm;

	pm = new_pixmap(imageWidth, imageHeight);

	for(y = 0; y < imageHeight; y++)
	{
  		for(x = 0; x < imageWidth; x++)
		{

			colour col;
			getRGBMapValues(x, y, imageWidth,rgbMap,&col);
			placepoint_ppm(pm, x, y, col.r, col.g, col.b);
		}
	}

	writeImage(imageFileName, pm);
}



void generatePixmapFromEnumIntMap(char * filename, int imageWidth, int imageHeight, int * intMap)
{
	int x,y;
	pixmap* pm;

	pm = new_pixmap(imageWidth, imageHeight);

	for(y = 0; y < imageHeight; y++)
	{
  		for(x = 0; x < imageWidth; x++)
		{
			int intVal;
			intVal = getIntMapValue(x, y, imageWidth, intMap);
			
			colour intValueColourRepresentation;
			if(intVal == 0)
			{
				intValueColourRepresentation.r = 0.0;
				intValueColourRepresentation.g = 0.0;
				intValueColourRepresentation.b = 0.0;
			}
			else if(intVal == 1)
			{
				intValueColourRepresentation.r = GREEN_R;
				intValueColourRepresentation.g = GREEN_G;
				intValueColourRepresentation.b = GREEN_B;			
			}
			else if(intVal == 2)
			{
				intValueColourRepresentation.r = BLUE_R;
				intValueColourRepresentation.g = BLUE_G;
				intValueColourRepresentation.b = BLUE_B;			
			}
			else if(intVal == 3)
			{
				intValueColourRepresentation.r = DARK_RED_R;
				intValueColourRepresentation.g = DARK_RED_G;
				intValueColourRepresentation.b = DARK_RED_B;			
			}
			else if(intVal == 4)
			{
				intValueColourRepresentation.r = DARK_GREEN_R;
				intValueColourRepresentation.g = DARK_GREEN_G;
				intValueColourRepresentation.b = DARK_GREEN_B;			
			}
			else if(intVal == 5)
			{
				intValueColourRepresentation.r = DARK_BLUE_R;
				intValueColourRepresentation.g = DARK_BLUE_G;
				intValueColourRepresentation.b = DARK_BLUE_B;			
			}
			else if(intVal == 6)
			{
				intValueColourRepresentation.r = YELLOW_R;
				intValueColourRepresentation.g = YELLOW_G;
				intValueColourRepresentation.b = YELLOW_B;			
			}
			else if(intVal == 7)
			{
				intValueColourRepresentation.r = CYAN_R;
				intValueColourRepresentation.g = CYAN_G;
				intValueColourRepresentation.b = CYAN_B;			
			}
			else if(intVal == 8)
			{
				intValueColourRepresentation.r = MAGENTA_R;
				intValueColourRepresentation.g = MAGENTA_G;
				intValueColourRepresentation.b = MAGENTA_B;			
			}
			else
			{
				//error;
				intValueColourRepresentation.r = RED_R;
				intValueColourRepresentation.g = RED_G;
				intValueColourRepresentation.b = RED_B;			
			}
			
			placepoint_ppm(pm, x, y, intValueColourRepresentation.r, intValueColourRepresentation.g, intValueColourRepresentation.b);
		}
	}

	writeImage(filename, pm);
}









void generatePixmapFromLuminosityContrastMap(char * imageFileName, int imageWidth, int imageHeight, double * contrastMap)
{
	int x,y;
	pixmap* pm;

	pm = new_pixmap(imageWidth, imageHeight);

	for(y = 0; y < imageHeight; y++)
	{
  		for(x = 0; x < imageWidth; x++)
		{
			double contrast;
			contrast = getLumOrContrastOrDepthMapValue(x, y, imageWidth, contrastMap);
			int contrastNormalised = minInt((int)(contrast/MAX_LUMINOSITY_CONTRAST*MAX_RGB_VAL), MAX_RGB_VAL);
		#ifdef USE_CONTRAST_PIXMAP_INVERSE
			contrastNormalised = -(contrastNormalised-MAX_RGB_VAL);
		#endif
			placepoint_ppm(pm, x, y, contrastNormalised, contrastNormalised, contrastNormalised);
		}
	}

	writeImage(imageFileName, pm);
}

void generateRGBMapFromDepthMapOrDepthContrastMap(int imageWidth, int imageHeight, double * depthMap, unsigned char * rgbMap)
{
	for(int y = 0; y < imageHeight; y++)
	{
  		for(int x = 0; x < imageWidth; x++)
		{
			double tAtSurface;
			tAtSurface = getLumOrContrastOrDepthMapValue(x, y, imageWidth, depthMap);
			
			//printf("\n %f", tAtSurface);
			
			if(tAtSurface == RT_RAYTRACE_NO_HIT_DEPTH_T)
			{	
				tAtSurface = ESTIMATE_MAX_DEPTH_T_REAL;
			}
			int tValSurfaceNormalised = minInt((int)(tAtSurface/ESTIMATE_MAX_DEPTH_T_REAL*MAX_RGB_VAL), MAX_RGB_VAL);
		#ifdef USE_CONTRAST_PIXMAP_INVERSE
			tValSurfaceNormalised = -(tValSurfaceNormalised-MAX_RGB_VAL);
		#endif
			
			colour col;
			col.r = tValSurfaceNormalised;
			col.g = tValSurfaceNormalised;
			col.b = tValSurfaceNormalised;
			setRGBMapValues(x, y, imageWidth, &col, rgbMap);
		}
	}
}

void generateRGBMapFromDepthMapOrDepthContrastMapAdvanced(int imageWidth, int imageHeight, double * depthMap, unsigned char * rgbMap, double noHitDepth, double estimateMaxDepth)
{
	for(int y = 0; y < imageHeight; y++)
	{
  		for(int x = 0; x < imageWidth; x++)
		{
			double tAtSurface;
			tAtSurface = getLumOrContrastOrDepthMapValue(x, y, imageWidth, depthMap);
			
			//printf("\n %f", tAtSurface);
			
			if(tAtSurface == noHitDepth)
			{	
				tAtSurface = estimateMaxDepth;
			}
			int tValSurfaceNormalised = minInt((int)(tAtSurface/estimateMaxDepth*MAX_RGB_VAL), MAX_RGB_VAL);
		#ifdef USE_CONTRAST_PIXMAP_INVERSE
			tValSurfaceNormalised = -(tValSurfaceNormalised-MAX_RGB_VAL);
		#endif
			
			colour col;
			col.r = tValSurfaceNormalised;
			col.g = tValSurfaceNormalised;
			col.b = tValSurfaceNormalised;
			setRGBMapValues(x, y, imageWidth, &col, rgbMap);
		}
	}
}



void generatePixmapFromDepthMapOrDepthContrastMap(char * imageFileName, int imageWidth, int imageHeight, double * depthMap)
{
	int x,y;
	pixmap* pm;

	pm = new_pixmap(imageWidth, imageHeight);

	for(y = 0; y < imageHeight; y++)
	{
  		for(x = 0; x < imageWidth; x++)
		{
			
			
			double tAtSurface;
			tAtSurface = getLumOrContrastOrDepthMapValue(x, y, imageWidth, depthMap);
			
			//temp;
			/*
			if(strcmp(imageFileName, "cubeinitialViewMapViewIndex0.depth.train.ppm") == 0)
			{
				printf("\n %f", tAtSurface);
			}
			*/
			
			if(tAtSurface == RT_RAYTRACE_NO_HIT_DEPTH_T)
			{	
				tAtSurface = ESTIMATE_MAX_DEPTH_T_REAL;
			}
			int tValSurfaceNormalised = minInt((int)(tAtSurface/ESTIMATE_MAX_DEPTH_T_REAL*MAX_RGB_VAL), MAX_RGB_VAL);
		#ifdef USE_CONTRAST_PIXMAP_INVERSE
			tValSurfaceNormalised = -(tValSurfaceNormalised-MAX_RGB_VAL);
		#endif
			placepoint_ppm(pm, x, y, tValSurfaceNormalised, tValSurfaceNormalised, tValSurfaceNormalised);
		}
	}

	writeImage(imageFileName, pm);
}

void generatePixmapFromDepthMapOrDepthContrastMapAdvanced(char * imageFileName, int imageWidth, int imageHeight, double * depthMap, double noHitDepth, double estimateMaxDepth)
{
	int x,y;
	pixmap* pm;

	pm = new_pixmap(imageWidth, imageHeight);

	for(y = 0; y < imageHeight; y++)
	{
  		for(x = 0; x < imageWidth; x++)
		{
			
			
			double tAtSurface;
			tAtSurface = getLumOrContrastOrDepthMapValue(x, y, imageWidth, depthMap);
			
			//printf("\n %f", tAtSurface);

			if(tAtSurface == noHitDepth)
			{	
				tAtSurface = estimateMaxDepth;
			}
			int tValSurfaceNormalised = minInt((int)(tAtSurface/estimateMaxDepth*MAX_RGB_VAL), MAX_RGB_VAL);
		#ifdef USE_CONTRAST_PIXMAP_INVERSE
			tValSurfaceNormalised = -(tValSurfaceNormalised-MAX_RGB_VAL);
		#endif
			placepoint_ppm(pm, x, y, tValSurfaceNormalised, tValSurfaceNormalised, tValSurfaceNormalised);
		}
	}

	writeImage(imageFileName, pm);
}



void generateRGBMapFromNormalMap(int imageWidth, int imageHeight, double * normalMap, unsigned char * rgbMap)
{		
	for(int y = 0; y < imageHeight; y++)
	{
  		for(int x = 0; x < imageWidth; x++)
		{
			vec norm;
			getNormalMapValue(x, y, imageWidth,normalMap,&norm);	
			
			colour col;
			col.r = (int)(norm.x*MAX_RGB_VAL);			//(int)((norm.x+1.0)*128.0)
			col.g = (int)(norm.y*MAX_RGB_VAL);			//(int)((norm.y+1.0)*128.0)
			col.b = (int)(norm.z*MAX_RGB_VAL);			//(int)((norm.z+1.0)*128.0)
			setRGBMapValues(x, y, imageWidth, &col, rgbMap);
						
		}
	}
}


void generatePixmapFromNormalMap(char * imageFileName, int imageWidth, int imageHeight, double * normalMap)
{		
	int x,y;
	pixmap* pm;

	pm = new_pixmap(imageWidth, imageHeight);

	for(y = 0; y < imageHeight; y++)
	{
  		for(x = 0; x < imageWidth; x++)
		{
			vec norm;
			getNormalMapValue(x, y, imageWidth,normalMap,&norm);				
			placepoint_ppm(pm, x, y, (int)(norm.x*MAX_RGB_VAL), (int)(norm.y*MAX_RGB_VAL), (int)(norm.z*MAX_RGB_VAL));
			//placepoint_ppm(nm, x, y, (int)((norm.x+1.0)*128.0), (int)((norm.y+1.0)*128.0), (int)((norm.z+1.0)*128.0));
		}
	}

	writeImage(imageFileName, pm);
}

void generatePixmapFromLuminosityMap(char * imageFileName, int imageWidth, int imageHeight, double * luminosityMap)
{
	int x,y;
	pixmap* pm;

	pm = new_pixmap(imageWidth, imageHeight);

	for(y = 0; y < imageHeight; y++)
	{
  		for(x = 0; x < imageWidth; x++)
		{
			double luminosity;
			luminosity = getLumOrContrastOrDepthMapValue(x, y, imageWidth, luminosityMap);
			int luminosityNormalised = minInt((int)(luminosity/MAX_LUMINOSITY*MAX_RGB_VAL), MAX_RGB_VAL);
		#ifdef USE_CONTRAST_PIXMAP_INVERSE
			luminosityNormalised = -(luminosityNormalised-MAX_RGB_VAL);
		#endif
			placepoint_ppm(pm, x, y, luminosityNormalised, luminosityNormalised, luminosityNormalised);
		}
	}

	writeImage(imageFileName, pm);
}



void generatePixmapFromBooleanMap(char * imageFileName, int imageWidth, int imageHeight, bool * booleanMap)
{
	int x,y;
	pixmap* pm;

	pm = new_pixmap(imageWidth, imageHeight);

	for(y = 0; y < imageHeight; y++)
	{
  		for(x = 0; x < imageWidth; x++)
		{
		#ifdef USE_BOOLEAN_PIXMAP_DOT_ENHANCEMENT
			#ifdef USE_BOOLEAN_PIXMAP_INVERSE
				int r = 255;
				int g = 255;
				int b = 255;
			#else
				int r = 0;
				int g = 0;
				int b = 0;		
			#endif
				placepoint_ppm(pm, x, y, r, g, b);
		#else
			bool val = getBooleanMapValue(x, y, imageWidth, booleanMap);
			int r;
			int g;
			int b;

			if(val == true)
			{
			#ifdef USE_BOOLEAN_PIXMAP_INVERSE
				r = 255;
				g = 255;
				b = 255;
			#else
				r = 0;
				g = 0;
				b = 0;  	    
			#endif
		
				placepoint_ppm(pm, x, y, 255, 255, 255);
			}
			else
			{
			#ifdef USE_BOOLEAN_PIXMAP_INVERSE
				r = 0;
				g = 0;
				b = 0;
			#else
				r = 255;
				g = 255;
				b = 255;  	    
			#endif
				placepoint_ppm(pm, x, y, 0, 0, 0);
			}			
			
		#endif
			
		}
	}

#ifdef USE_BOOLEAN_PIXMAP_DOT_ENHANCEMENT	
	for(y = 0; y < imageHeight; y++)
	{
  		for(x = 0; x < imageWidth; x++)
		{
			bool val;
			val = getBooleanMapValue(x, y, imageWidth, booleanMap);
			int r = 0;
			int g = 0;
			int b = 0;
			if(val == true)
			{
				r = BOOLEAN_PIXMAP_DOT_RED;
				g = BOOLEAN_PIXMAP_DOT_GREEN;
				b = BOOLEAN_PIXMAP_DOT_BLUE;
				for(int ky = y-BOOLEAN_PIXMAP_DOT_ENHANCEMENT; ky <= y+BOOLEAN_PIXMAP_DOT_ENHANCEMENT; ky++)
				{
					for(int kx = x-BOOLEAN_PIXMAP_DOT_ENHANCEMENT; kx <= x+BOOLEAN_PIXMAP_DOT_ENHANCEMENT; kx++)
					{
						if((ky >= 0) && (ky < imageHeight))
						{
							if((kx >= 0) && (kx < imageWidth))
							{
								placepoint_ppm(pm, kx, ky, r, g, b);
							}
						}
					}				
				}
			}	
		}
	}
#endif

	writeImage(imageFileName, pm);
}


void writeImage(char * imageFileName, pixmap * pm)
{
	write_ppm(imageFileName, pm);
	/*write_ppm(NULL, pm);*/ /*For Clinton*/ 
	free_pixmap(pm);	
}






















//methods taken from ORoperations.cpp;

void createRGBMapFromPixmapImage(pixmap * objectImage, unsigned char * rgbMap)
{
		
	int width = objectImage->wide;
	//cout << "4aii" << endl;
	int height =  objectImage->high;
	
	//fill luminosityMap
	for(int y = 0; y < height; y++)
	{
		for(int x = 0; x < width; x++)
		{
			int pixelPositionInPixmap = y*(objectImage->wide)*3+x*3;
			colour col;
			calculateColourFromPixmapPixel(objectImage, pixelPositionInPixmap, &col);
			//cout << "col.r = " << col.r << endl;
			setRGBMapValues(x, y, width, &col, rgbMap);
		}	
	}
}


void calculateColourFromPixmapPixel(pixmap * pm, int pixelPositionInPixmap, colour * col)
{
	
	col->r = pm->pix[(int)pixelPositionInPixmap];
	col->g = pm->pix[(int)pixelPositionInPixmap+1];
	col->b = pm->pix[(int)pixelPositionInPixmap+2];
}



void createLuminosityMapFromPixmapImage(pixmap * objectImage, double * luminosityMap)
{	
	int width = objectImage->wide;
	//cout << "4aii" << endl;
	int height =  objectImage->high;
	
	//fill luminosityMap
	for(int y = 0; y < height; y++)
	{
		for(int x = 0; x < width; x++)
		{
			int pixelPositionInPixmap = y*(objectImage->wide)*3+x*3;
			setLumOrContrastOrDepthMapValue(x, y, width, calculateLuminosityLevelFromPixmapPixel(objectImage, pixelPositionInPixmap), luminosityMap);

		}	
	}
}

double calculateLuminosityLevelFromPixmapPixel(pixmap * pm, int pixelPositionInPixmap)
{
	double luminosity;
	
	unsigned char r = pm->pix[(int)pixelPositionInPixmap];
	unsigned char g = pm->pix[(int)pixelPositionInPixmap+1];
	unsigned char b = pm->pix[(int)pixelPositionInPixmap+2];
	
	luminosity = (r+g+b);
	//cout << "luminosity = " << luminosity << endl;
	
	return luminosity;
}


void createLuminosityMapFromRGBMap(int width, int height, unsigned char * rgbMap, double * luminosityMap)
{	
	//fill luminosityMap
	for(int y = 0; y < height; y++)
	{
		for(int x = 0; x < width; x++)
		{
			double luminosityLevel = calculateLuminosityLevelFromRGBMap(x, y, width, rgbMap);
			setLumOrContrastOrDepthMapValue(x, y, width, luminosityLevel, luminosityMap);
		}	
	}
}

double calculateLuminosityLevelFromRGBMap(int x, int y, int imageWidth, unsigned char * rgbMap)
{
	double luminosity;
	
	#ifdef OLDcalculateLuminosityLevelFromRGBMap
		unsigned char r = getRGBMapValue(x, y, imageWidth, RGB_RED, rgbMap);
		unsigned char g = getRGBMapValue(x, y, imageWidth, RGB_GREEN, rgbMap);
		unsigned char b = getRGBMapValue(x, y, imageWidth, RGB_BLUE, rgbMap);
		luminosity = (r+g+b);
		//cout << "luminosity = " << luminosity << endl;	
	#else
		colour col;
		getRGBMapValues(x, y, imageWidth, rgbMap, &col);
		luminosity = calculateLuminosityLevelFromRGBVal(&col);
	#endif

	
	return luminosity;
}

double calculateLuminosityLevelFromRGBVal(colour * rgbVal)
{
	double luminosity;
	
	unsigned char r = rgbVal->r;
	unsigned char g = rgbVal->g;
	unsigned char b = rgbVal->b;
	
	luminosity = (r+g+b);
	//cout << "luminosity = " << luminosity << endl;
	
	return luminosity;
}





	//WARNING; no depth check!
void createLuminosityContrastMapFromRGBMap(int imageWidth, int imageHeight, unsigned char * rgbMap, double * luminosityContrastMap)
{
	double * luminosityMap = new double[imageWidth*imageHeight];
	
	createLuminosityMapFromRGBMap(imageWidth, imageHeight, rgbMap, luminosityMap);

	createContrastMapFromMap(imageWidth, imageHeight, luminosityMap, luminosityContrastMap);

	delete luminosityMap;
}


void createContrastMapFromMap(int width, int height, double * luminosityMap, double * contrastMap)
{
	//now lead a 3x3 kernel across image and calculate areas of high contrast
	for(int y = 0; y < height; y++)
	{
		for(int x = 0; x < width; x++)
		{
			//contrastMap[y*width + x] = calculateContrastLevelWithinKernel(x, y, luminosityMap, CONTRAST_MAP_GENERATION_KERNEL_WIDTH, CONTRAST_MAP_GENERATION_KERNEL_HEIGHT, width, height);
			double contrastVal = calculateContrastLevelWithinKernel(x, y, luminosityMap, CONTRAST_MAP_GENERATION_KERNEL_WIDTH, CONTRAST_MAP_GENERATION_KERNEL_HEIGHT, width, height);
			setLumOrContrastOrDepthMapValue(x, y, width, contrastVal, contrastMap);
		}	
	}
}


double calculateContrastLevelWithinKernel(int pixelX, int pixelY, double * luminosityMap, int kernelWidth, int kernelHeight, int imageWidth, int imageHeight)
{		
	double contrastLevel = 0.0;

	double centrePixelPositionInLummapLuminosity = getLumOrContrastOrDepthMapValue(pixelX, pixelY, imageWidth, luminosityMap);

	for(int y = pixelY-(kernelHeight/2); y<= pixelY+(kernelHeight/2); y++)	
	{
		for(int x = pixelX-(kernelWidth/2); x<= pixelX+(kernelWidth/2); x++)
		{
			if((y >= 0) && (y <imageHeight))
			{//check kernel pix is within image
				
				if((x >= 0) && (x <imageWidth))
				{//check kernel pix is within image
			
					if((x == pixelX) && (y == pixelY))
					{
					
					}
					else
					{
						//cout << "x = " << x << endl;
						//cout << "y = " << y << endl;
						
						//calc diff lum diff between centre pixel and current surrounding kernel pixel

						double kernelCurrentPixelPositionInLummapLuminosity = getLumOrContrastOrDepthMapValue(x, y, imageWidth, luminosityMap);
						double currentContrastLevel = absDouble(centrePixelPositionInLummapLuminosity - kernelCurrentPixelPositionInLummapLuminosity);
						contrastLevel = maxDouble(contrastLevel, currentContrastLevel);
						//cout << "contrastLevel = " << contrastLevel << endl;
						 
						/*	//do not apply threshold for contrast levels here!
						#define COLOUR_SATURATION_DIFF_FOR_CONTRAST_EDGE 30
						int centrePixelPositionInPixmap = pixelY*(objectImage->wide)*3+pixelX*3;
						int kernelCurrentPixelPositionInPixmap = y*(objectImage->wide)*3+x*3;
						for(int col=0; col<3; col++)
						{

							int centrePixelPositionInPixmapColour = objectImage->pix[(int)centrePixelPositionInPixmap+col];
							int kernelCurrentPixelPositionInPixmapColour = objectImage->pix[(int)kernelCurrentPixelPositionInPixmapColour+col];
							if(abs(centrePixelPositionInPixmapColour - kernelCurrentPixelPositionInPixmapColour) > COLOUR_SATURATION_DIFF_FOR_CONTRAST_EDGE)
							{
								contrastLevel = max			
							}

						}
						*/
					}
				}
			}
		}	
	}
	
	return contrastLevel;
}










void createLuminosityContrastBooleanMap(int imageWidth, int imageHeight, double * luminosityContrastMap, bool * luminosityContrastBooleanMap)
{						
	for(int y = 0; y < imageHeight; y++)
	{
		for(int x = 0; x < imageWidth; x++)
		{
			double luminosityContrastVal = getLumOrContrastOrDepthMapValue(x, y, imageWidth, luminosityContrastMap);
			bool luminosityContrastValPassedThreshold;
			if(luminosityContrastVal > MAX_LUMINOSITY_CONTRAST*LUMINOSITY_CONTRAST_FRACTION_THRESHOLD)
			{
				luminosityContrastValPassedThreshold = true;
			} 
			else
			{
				luminosityContrastValPassedThreshold = false;
			}
			setBooleanMapValue(x, y, imageWidth, luminosityContrastValPassedThreshold, luminosityContrastBooleanMap);
		}
	}
}

void createLuminosityBooleanMap(int imageWidth, int imageHeight, double * luminosityMap, bool * luminosityBooleanMap)
{						
	for(int y = 0; y < imageHeight; y++)
	{
		for(int x = 0; x < imageWidth; x++)
		{
			double luminosityVal = getLumOrContrastOrDepthMapValue(x, y, imageWidth, luminosityMap);
			bool luminosityValPassedThreshold;
			if(luminosityVal > MAX_LUMINOSITY*LUMINOSITY_FRACTION_THRESHOLD)
			{
				luminosityValPassedThreshold = true;
			} 
			else
			{
				luminosityValPassedThreshold = false;
			}
			setBooleanMapValue(x, y, imageWidth, luminosityValPassedThreshold, luminosityBooleanMap);
		}
	}
}






