/*******************************************************************************
 *
 * File Name: SHAREDglobalDefs.h
 * Author: Richard Baxter
 * Project: Generic Construct Functions
 * Project Version: 2a4d 24-Oct-09 [1n22a; OR_TEST_ORI_AND_POS_NOISE_DURING_TRANING_AND_TESTING]
 *
 * System Requirements; FD.exe "heitger feature detection with RBB Endian Mod v0.5b"
 *							olof Henricsson 96-06-07 release of Friedrich Heitger's SE binary with 
 *							a) ras format endian swap, b) ras format relaxed, 
 *							c) multiple endian outputs for Keypoints exe, d) Windows VS compatibility, 
 *							e) change of binary name to FD.exe
 *			ImageMagik
 *			Freeglut (for hardware accelerated opengl rendering)
 *	
 * Important vars to consider before compilation;
 *			LINUX 
 *			OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2 versus OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO
 *
 * feature.cpp;
 * 1. need to make sure heitger feature detection will work with a contrast map instead of an rgb map (or more specifically, with a point normal contrast map)
 *
 * 2.	#ifdef OR_USE_FIND_CONTIGUOUS_REGION_CENTRED_FEATURES_BOUNDARY_FEATURES
 *	//find CENTRED_FEATURES_BOUNDARY_FEATURES... [not yet coded]
 *	#endif
 *******************************************************************************/

#ifndef HEADER_SHARED_GLOBAL_DEFS
#define HEADER_SHARED_GLOBAL_DEFS



	/**********
	TEMPORARY TEST VARIABLES [LATEST DEVELOPMENT CYCLE];
	***********/

#define TEST_GEO		//temp (reduces number of objects and view points for software testing)
//#define OR_QUADRATIC_FIT_DEBUG

//all the following are seriously untested [comment out TRY_UNTESTED_SOFTWARE for proper/formal/release software build]
#define TRY_UNTESTED_SOFTWARE
#ifdef TRY_UNTESTED_SOFTWARE
	//#define OLDcalculateLuminosityLevelFromRGBMap
	#define RT_CONVERT_LDR_OBJECT_SCALES_PROPERLY
	#define RT_CONVERT_LDR_OBJECT_ROTATION_PROPERLY
	//#define RT_CONVERT_LDR_OBJECT_TRANSLATION_PROPERLY
	#define RT_CONVERT_LDR_OBJECT_TRANSLATION_PROPERLY_VERSIONB (untested - may overcome no draw centre pixel line/column of object issue with RT_CONVERT_LDR_OBJECT_TRANSLATION_PROPERLY)



	//find and use centred features
	#define OR_USE_CONTIGUOUS_REGION_FIND_CENTRED_FEATURES
	#define OR_TRACE_CONTIGUOUS_REGION_BOUNDARY_EDGE_EXACTLY_INSTEAD_OF_OUTER_CONTIGUOUS_REGION_BOUNDARY	//this is a far more accurate method
	#ifdef OR_TRACE_CONTIGUOUS_REGION_BOUNDARY_EDGE_EXACTLY_INSTEAD_OF_OUTER_CONTIGUOUS_REGION_BOUNDARY
		#define OR_USE_FIND_CONTIGUOUS_REGION_CENTRED_FEATURES_BOUNDARY_FEATURES 
	#endif
	#define OR_METHOD3DOD_USE_3D_EDGE_POINTS_CALCULATED_DURING_QUADRATIC_FIT_INSTEAD_OF_DURING_EDGE_TRACE	//this will not be used in the future as edge points calculated will have depth filtering applied based upon contiguous boundary edge trace 
	
	#define MIN_REGION_SIZE_TO_CALCULATE_CENTRED_FEATURE (20)
	#define OR_METHOD_3DOD_CONTINUOUS_EDGE_MAX_NOISE (2)	//this needs to be made dynamic based upon vi
	#define OR_METHOD_3DOD_CONTINUOUS_EDGE_MAX_POINT_MOVEMENT (0.1)	//this needs to be made dynamic based upon vi
	#define OR_METHOD_3DOD_CONTINUOUS_EDGE_MAX_NOISE_2 (OR_METHOD_3DOD_CONTINUOUS_EDGE_MAX_NOISE)	//this needs to be checked, this value might need to be increased slightly
	
	//mesh lists (only currently used for centred feature calculations in combined mesh lists)
	#define OR_METHOD_USE_MESH_LISTS
	#ifdef OR_METHOD_USE_MESH_LISTS
		#define OR_METHOD_3DOD_USE_MESH_LISTS_COMBINED	//probably need to verify quadratic fit edges are calculated properly upon mesh advanced join
		#define OR_METHOD_3DOD_USE_ADVANCED_INTERP_MESH_JOINING_MAXIMUM_RECONCILIATION_DISTANCE (0.1) //this value needs to be dynamic
		//#define OR_METHOD_2DOD_USE_MESH_LISTS_COMBINED		//dont use	//not yet implemented (low priority) - there is no way of combining 2D mesh lists without eg x/y offset information...
	#endif
	
	//mesh list quadratic fit for accurate edge calculation and centre feature calculations
	#define OR_METHOD3DOD_USE_QUADRATIC_FIT_EDGE_ZERO_CROSSING_MAP
	#ifdef OR_METHOD3DOD_USE_QUADRATIC_FIT_EDGE_ZERO_CROSSING_MAP
		#define OR_METHOD_QUADRATIC_FIT_FOR_MESH_LISTS_HAS_BEEN_PROGRAMMED
		#define OR_METHOD3DOD_USE_QUADRATIC_FIT_EDGE_ZERO_CROSSING_MAP_BOOL (true)
	#else
		#define OR_METHOD3DOD_USE_QUADRATIC_FIT_EDGE_ZERO_CROSSING_MAP_BOOL (false)
	#endif
	#define OR_METHOD2DOD_USE_QUADRATIC_FIT_EDGE_ZERO_CROSSING_MAP
	#ifdef OR_METHOD2DOD_USE_QUADRATIC_FIT_EDGE_ZERO_CROSSING_MAP
		#define OR_METHOD_QUADRATIC_FIT_FOR_MESH_LISTS_HAS_BEEN_PROGRAMMED
		#define OR_METHOD2DOD_USE_QUADRATIC_FIT_EDGE_ZERO_CROSSING_MAP_BOOL (true)
	#else
		#define OR_METHOD2DOD_USE_QUADRATIC_FIT_EDGE_ZERO_CROSSING_MAP_BOOL (false)
	#endif
	
	#define OR_METHOD3DOD_USE_MESH_NORMAL_AND_NORMAL_CONTRAST	//not currently required or used
	
	//this is an update which gets rid of depth gradient contrast and depth contrast maps
	#define OR_METHOD3DOD_USE_NORMAL_AND_NORMAL_CONTRAST_INSTEAD_OF_DEPTH_GRADIENT_AND_DEPTH_GRADIENT_CONTRAST_FOR_3D_CORNER_CALCULATIONS
	
	
		
	//the above is only valid when OR_METHOD_3DOD_ASSUME_TRAIN_AND_TEST_WITH_SINGLE_VIEWPOINT is not defined
			
	
	
	
	//#define OR_USE_ADVANCED_REF_INTERP_MESH_JOINING_INSTEAD_OF_BASIC_OVERLAY	//not yet implemented (low priority) - reference lists or display mesh should be able to be overlayed without major issues - NB if issues are found we must implement advanced mesh joining for these reference lists
	
	
#else
	#define RT_CONVERT_LDR_OBJECT_SCALES_PROPERLY
	#define RT_CONVERT_LDR_OBJECT_ROTATION_PROPERLY
	#define RT_CONVERT_LDR_OBJECT_TRANSLATION_PROPERLY
#endif



	/**********
	OR VARIABLES TESTED BUT NOT YET COMMISSIONED;
	***********/
	
//#define OR_METHOD_3DOD_USE_DYNAMIC_WORLD_COORD_DETERMINATION_USING_DEPTH	


	/**********
	OR VARIABLES NOT YET TESTED;
	***********/
	
//#define TRYTHISOUT
//#define OR_METHOD_3DOD_USE_RGBMAP_FOR_FEAT_DET_ALSO				//not yet tested




//in order of current compilation preference;

	/**********
	SOURCE PROJECT VARIABLES;
	***********/

		//current/active tests;
	//#define COMPILE_TH_OR_METHOD_2DOD
	//#define COMPILE_TH_OR_METHOD_3DOD
	//#define COMPILE_FD 
	#define COMPILE_CS
	//#define COMPILE_RT
	//#define COMPILE_ANN_WITH_RT
	//#define COMPILE_ANN

		//inactive tests;
	//#define COMPILE_LRRC_WITH_ANN
	//#define COMPILE_TH_ANN_WITH_LRRC
	//#define COMPILE_LRRC
	//#define COMPILE_OLC

		//old tests
	//#define COMPILE_TH_RT
	//#define COMPILE_TH_OR_METHOD_2DOD_WITH_ANN
	//#define COMPILE_TH_OR_METHOD_3DOD_WITH_ANN


	/**********
	OS DEPENDENT VARIABLES;
	***********/


#define LINUX	
			//else assume windows (effects default parts directory only - this is ISO C++ so it should compile on any platform)
#define IMAGEMAGIK_INSTALLED
#ifndef LINUX
	//#include <windows.h>
#endif




	/**********
	OR TIME TEST VARIABLES;
	***********/

//#define TEST_VIEWTIME_SPEED_Z_FOR_WEBSITE
//#define TEST_VIEWTIME_SPEED_Z_FOR_WEBSITE_OLD_ADDITIONAL


#ifndef TEST_VIEWTIME_SPEED_Z_FOR_WEBSITE
	#define OR_GENERATE_IMAGE_COMPARITOR_RESULTS_HTML
#endif



	/**********
	3DOD OR VARIABLES NOT YET FULLY IMPLEMENTED [there is no depth map comparison currently being performed];
	***********/

#define OR_METHOD_3DOD_USE_SNAPSHOT_DEPTH_MAPS
	#define UNKNOWN_OPENGL_NOHIT_DEPTH (0.0)
	#define UNKNOWN_OPENGL_ESTIMATE_MAX_DEPTH (1.0)
	//#define FIX_OPENGL_3DOD_TIMING_BUG
	

	/**********
	OR DEBUG VARIABLES;
	***********/

	//debug only;
#define OR_METHOD_GEO_COMPARISON_RECORD_ORIGINAL_T_FOR_DEBUG


	/**********
	OR FEATURE ACCURACY VARIABLES;
	***********/
	
#define USE_RBB_QUADRATIC_FIT_KEYPOINT_GENERATION		//best accuracy
	//#define OR_QUADRATIC_FIT_DEBUG
//#define USE_RBB_BASIC_KEYPOINT_GENERATION			//old inaccurate
//#define USE_HEITGER_ASCII_KEYPOINT_GENERATION			//old inaccurate



	/**********
	OR COMMON VARIABLES;
	***********/
	
#define OR_METHOD_POLYGON_NUMBER_OF_SIDES (3) //triangle


	/**********
	OR TRANSFORMATION ACCURACY VARIABLES 
	***********/
	
#define OR_METHOD_3DOD_USE_POLYGON_MIN_MAX_INTERNAL_ANGLE_TEST
#define OR_METHOD_2DOD_USE_POLYGON_MIN_MAX_INTERNAL_ANGLE_TEST
#define POLYGON_MIN_ANGLE_DEGREES (10.0)

#define OR_METHOD3DOD_MINIMUM_AREA_OF_NORMALISATION_POLYGON (20.0)	//this needs to be made dynamic in the future! [based upon distance object is away from pov]
#define OR_METHOD2DOD_MINIMUM_AREA_OF_NORMALISATION_POLYGON (20.0)	//100 //20
//#define OR_USE_BAD_ADVANCED_AREA_OF_T_TEST


	/**********
	OR SCALING VARIABLES 
	***********/

#ifdef COMPILE_TH_OR_METHOD_2DOD
	#define OR_METHOD_SUPPORT_HIGH_LEVEL_SCALING
	#ifdef OR_METHOD_SUPPORT_HIGH_LEVEL_SCALING
		#define OR_METHOD_SUPPORT_SNAPSHOTS_AT_MULTIPLE_ZOOMS
		#define OR_METHOD_NUMBER_OF_SNAPSHOT_ZOOM_LEVELS
	#endif
#endif

	/**********
	OR PERFORMANCE FOR SPEED SACRIFICE VARIABLES 
	***********/
	
	//3. optimisation: image comparison hue binning [will currently theoretically provides a small speed boost, but will limit OR to environments where lighting hue does not vary between train and test]
//#define OR_METHOD_DO_NOT_REQUIRE_TO_SUPPORT_LIGHTING_COLOUR_HUE_VARIATION	
#ifdef OR_METHOD_DO_NOT_REQUIRE_TO_SUPPORT_LIGHTING_COLOUR_HUE_VARIATION
	#define OR_METHOD_SMALL_IMAGE_FIRST_COMPARISON_NO_KERNEL
	#define OR_METHOD_IMAGE_COMPARISON_BINNING_BASIC
	#ifdef OR_METHOD_IMAGE_COMPARISON_BINNING_BASIC		//currently only works with OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO [not OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2]!!!!!!
		//#define OR_METHOD_IMAGE_COMPARISON_BINNING_ADVANCED	(2x2 pixel image segmentations)
		#define OR_METHOD_IMAGE_COMPARISON_BINNING_NUM_DISTINCT_VALS_PER_COL (20)
		#define OR_METHOD_IMAGE_COMPARISON_BINNING_NO_BINS_TO_COMPARE (3)
		#define OR_METHOD_IMAGE_COMPARISON_BINNING_ALLOW_LIGHTING_COLOUR_SATURATION_VARIATION
		#ifdef OR_METHOD_IMAGE_COMPARISON_BINNING_ALLOW_LIGHTING_COLOUR_SATURATION_VARIATION
			#define OR_METHOD_IMAGE_COMPARISON_BINNING_LIGHTING_COLOUR_SATURATION_VARIATION_VAL_PER_COL (10)
		#endif
	#endif
#endif



	/**********
	OR OPTIMISATION VARIABLES;
	***********/


	//1. optimisation: image cropping
	
#ifdef COMPILE_TH_OR_METHOD_2DOD
	#define OR_METHOD_USE_NORM_SNAPSHOT_CROP	//CURRENTLY REQUIRES OPENGL!
#endif
#ifdef OR_METHOD_USE_NORM_SNAPSHOT_CROP
	#define OR_METHOD_NORM_SNAPSHOT_CROP_X (11)	//this can be 11 instead of 12
	#define OR_METHOD_NORM_SNAPSHOT_CROP_Y (11)	//this can be 13 instead of 12
	#define OR_METHOD_NORM_SNAPSHOT_X (28)
	#define OR_METHOD_NORM_SNAPSHOT_Y (28)
	#define OR_METHOD_USE_SMALL_IMAGE_RATIO (4)

#else
	#define OR_METHOD_NORM_SNAPSHOT_CROP_X (0)
	#define OR_METHOD_NORM_SNAPSHOT_CROP_Y (0)
	#define OR_METHOD_NORM_SNAPSHOT_X (50)
	#define OR_METHOD_NORM_SNAPSHOT_Y (50)
	#define OR_METHOD_USE_SMALL_IMAGE_RATIO (5)
#endif

	//2. optimisation: small image comparison


#define OR_METHOD_USE_SMALL_IMAGE_FIRST_COMPARISON

//#define OR_METHOD_USE_SMALL_IMAGE_RATIO_IGNORE_BG_COMPARISON	//old
#define OR_METHOD_USE_SMALL_IMAGE_RATIO_IGNORE_BG_COMPARISON_V2	//new
//#define OR_METHOD_USE_SPECIFIC_BG_COLOUR
#ifdef OR_METHOD_USE_SPECIFIC_BG_COLOUR
	#define OR_SNAPSHOT_BACKGROUND_COLOUR_R (255)
	#define OR_SNAPSHOT_BACKGROUND_COLOUR_G (0)
	#define OR_SNAPSHOT_BACKGROUND_COLOUR_B (255)
	#define OR_SNAPSHOT_BACKGROUND_COLOUR_R_NORM (1.0)
	#define OR_SNAPSHOT_BACKGROUND_COLOUR_G_NORM (0.0)
	#define OR_SNAPSHOT_BACKGROUND_COLOUR_B_NORM (1.0)
#else
	#define OR_SNAPSHOT_BACKGROUND_COLOUR_R (0)
	#define OR_SNAPSHOT_BACKGROUND_COLOUR_G (0)
	#define OR_SNAPSHOT_BACKGROUND_COLOUR_B (0)
	#define OR_SNAPSHOT_BACKGROUND_COLOUR_R_NORM (0.0)
	#define OR_SNAPSHOT_BACKGROUND_COLOUR_G_NORM (0.0)
	#define OR_SNAPSHOT_BACKGROUND_COLOUR_B_NORM (0.0)
#endif
#define OR_IMAGE_COMPARITOR_SMALL_MATCH_ERROR_THRESHOLD (0.2)


	//4. optimisation: probably will never be implemented;
//#define OR_GEOMETRIC_CHECK_COMPARISON_ERROR_DEV_CHECK	//not yet tested
	//#define OR_GEOMETRIC_CHECK_COMPARISON_ERROR_DEV (0.2)


	//5. optimisation: not yet tested;
//#define OR_METHOD_GEO_COMPARISON_DYNAMIC_ERROR_THRESHOLD
//#define OR_METHOD_GEO_COMPARISON_DYNAMIC_ERROR_THRESHOLD_MIN_EXPECTED_ORIG_TRI_WIDTH_OR_HEIGHT (10)	//in pixels

	//6. optimisation: newly implemented;
//#define OR_DEBUG_METHOD_3DOD
#define OR_METHOD_3DOD_USE_FORMAL_TRANSFORMATION_METHOD		//when all issues related to this are resolved and this becomes the default, an inverse definition name should be implemented for clarity

	//7. optimisation: Geometric comparison (enabling this increases execution speed x1000, but decreases redundancy);
		//(if 3D, require OR_METHOD_3DOD_USE_FORMAL_TRANSFORMATION_METHOD)

//#define TEMPTEST3GEOACCURACY	//debug only;


#define OR_METHOD_TRANSFORM_NEARBY_FEATURES_AND_ENABLE_GEOMETRIC_COMPARISON

#ifdef OR_METHOD_TRANSFORM_NEARBY_FEATURES_AND_ENABLE_GEOMETRIC_COMPARISON
	#define OR_METHOD_CREATE_NEARBY_FEATURES_FILE
	#define OR_METHOD_GEOMETRIC_COMPARISON_ON
	#define OR_METHOD_GEOMETRIC_COMPARISON_VARIABLES
	#define OR_METHOD_GEOMETRIC_COMPARISON_VARIABLES_BASIC
	
#ifdef LINUX	
	//NB V2 option currently does not work with windows compilation	due to array dec differences between vis c++ and g++
	#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2	//was supposed to give higher performance than v1, but does not because nearest feature matching error is so high.. [possibly will work once feature detection has subpixel accuracy applied]	
	//#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO
#else
	//#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO
#endif
	
	#ifndef USE_RBB_QUADRATIC_FIT_KEYPOINT_GENERATION
		#define USE_GEO_TEST_GRID_SIZE_LARGE
	#endif
	#ifdef COMPILE_TH_OR_METHOD_3DOD
		//#ifndef OR_METHOD_3DOD_USE_DYNAMIC_WORLD_COORD_DETERMINATION_USING_DEPTH
			#define USE_GEO_TEST_GRID_SIZE_LARGE
		//#endif
	#endif
	#ifdef USE_GEO_TEST_GRID_SIZE_LARGE
		#define OR_GEOMETRIC_CHECK_COMPARISON_MAX_ERROR (0.5)
	#else
		#define OR_GEOMETRIC_CHECK_COMPARISON_MAX_ERROR (0.1)
	#endif
	
	
	#define OR_GEOMETRIC_CHECK_MIN_NUMBER_PASSES (2)	//default; 2

	#define OR_METHOD_TRANSFORM_NEARBY_FEATURES_TAG_OT_FEATURES
	#define NUMBER_OF_POLYGONS_PER_FEATURE (3) 		//default; 3
	#define OR_METHOD_NUM_NEARBY_FEATURES_TO_COMPARE (4)	//default; 4, 10 will reduce speed but increase redundancy [due to bug?]
	#ifdef COMPILE_TH_OR_METHOD_2DOD
		#define OR_METHOD_NUM_NEARBY_FEATURES_TO_TRANSFORM (OR_METHOD_NUM_NEARBY_FEATURES_TO_COMPARE)
	#elif defined COMPILE_TH_OR_METHOD32DOD
		#define OR_METHOD_NUM_NEARBY_FEATURES_TO_TRANSFORM (OR_METHOD_NUM_NEARBY_FEATURES_TO_COMPARE - OR_METHOD_POLYGON_NUMBER_OF_SIDES)	//with 3DOD, all features including those on the OT are compared, so less additional nearby features are required for transformation
	#else
		#define IRRELEVANT (1)
		#define OR_METHOD_NUM_NEARBY_FEATURES_TO_TRANSFORM (IRRELEVANT)
	#endif
	
	#define OR_METHOD_GEOMETRIC_COMPARISON_USE_TEST_RGB_MAP_OPTIMISATION

	#ifdef OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2
		#ifdef USE_GEO_TEST_GRID_SIZE_LARGE
			#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_NEAREST_FEATURE_MAX_DISTANCE (12.5)
		#else
			#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_NEAREST_FEATURE_MAX_DISTANCE (2.5)		//nearest features checked for geo hashing can lie within 8x the extent of the OT, assuming OT has a side diameter of 1
		#endif
		#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_X_BIN_RANGE (OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_NEAREST_FEATURE_MAX_DISTANCE*2)
		#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_Y_BIN_RANGE (OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_NEAREST_FEATURE_MAX_DISTANCE*2)
		#ifdef USE_GEO_TEST_GRID_SIZE_LARGE
			#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_X_BIN_SEPARATION (OR_GEOMETRIC_CHECK_COMPARISON_MAX_ERROR)
			#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_Y_BIN_SEPARATION (OR_GEOMETRIC_CHECK_COMPARISON_MAX_ERROR)
		#else
			#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_X_BIN_SEPARATION (OR_GEOMETRIC_CHECK_COMPARISON_MAX_ERROR)
			#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_Y_BIN_SEPARATION (OR_GEOMETRIC_CHECK_COMPARISON_MAX_ERROR)
		#endif

		#ifdef USE_GEO_TEST_GRID_SIZE_LARGE
			#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_NO_X_BINS (50)				//((long)(((double)OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_X_BIN_RANGE)/OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_X_BIN_SEPARATION))
			#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_NO_Y_BINS (50)				//((long)(((double)OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_X_BIN_RANGE)/OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_X_BIN_SEPARATION))
		#else
			#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_NO_X_BINS (50)				 //((long)(((double)OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_X_BIN_RANGE)/OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_X_BIN_SEPARATION))
			#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_NO_Y_BINS (50)				 //((long)(((double)OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_Y_BIN_RANGE)/OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_Y_BIN_SEPARATION))
		#endif
		#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_NO_BINS_TO_COMPARE_X (3)
		#define OR_METHOD_GEOMETRIC_COMPARISON_OPTIMISED_FILE_IO_V2_NO_BINS_TO_COMPARE_Y (3)


	#endif
#elif defined TEMPTEST3GEOACCURACY
	#define OR_GENERATE_IMAGE_COMPARITOR_RESULTS_HTML_PRINT_OBJECT_NAMES
	#define OR_METHOD_CREATE_NEARBY_FEATURES_FILE
	#define OR_METHOD_GEOMETRIC_COMPARISON_VARIABLES
	#define OR_METHOD_TRANSFORM_NEARBY_FEATURES_TAG_OT_FEATURES
	#define OR_METHOD_NUM_NEARBY_FEATURES_TO_COMPARE (4)
	#define OR_METHOD_NUM_NEARBY_FEATURES_TO_TRANSFORM (OR_METHOD_NUM_NEARBY_FEATURES_TO_COMPARE)
	#define NUMBER_OF_POLYGONS_PER_FEATURE (3)
	#define OR_GEOMETRIC_CHECK_COMPARISON_MAX_ERROR (0.2)
	#define OR_GEOMETRIC_CHECK_MIN_NUMBER_PASSES (2)
#else
	#define OR_METHOD_NUM_NEARBY_FEATURES_TO_TRANSFORM (0)
	#define NUMBER_OF_POLYGONS_PER_FEATURE (3) //default: 3
	#define OR_METHOD_GEOMETRIC_COMPARISON_USE_TEST_RGB_MAP_OPTIMISATION
#endif

#define OR_METHOD_TRANSFORM_KEY_OT_FEATURES
#ifdef OR_METHOD_TRANSFORM_KEY_OT_FEATURES
	#define OR_METHOD_CREATE_OT_FEATURES_FILE
	#define OR_METHOD_GEOMETRIC_COMPARISON_VARIABLES_BASIC
#endif
//#define OR_METHOD_TRANSFORM_ALL_FEATURES		//do not use this; it is inefficient
#ifdef OR_METHOD_TRANSFORM_ALL_FEATURES
	#define OR_METHOD_CREATE_ALL_FEATURES_FILE
	#define OR_METHOD_GEOMETRIC_COMPARISON_VARIABLES_BASIC
#endif




	/**********
	OR DISPLAY VARIABLES;
	***********/

//display important;
#define OR_GENERATE_IMAGE_COMPARITOR_RESULTS_NO_EXPLICIT_CONFIDENTIAL_WARNINGS
#define OR_GENERATE_IMAGE_COMPARITOR_RESULTS_ALLOW_CONFIDENTIAL
#define OR_GENERATE_IMAGE_COMPARITOR_RESULTS_HTML_PRINT_OBJECT_NAMES

//display general;
#define OR_PRINT_ALGORITHM_PROGRESS
#define OR_PRINT_ALGORITHM_AND_TIME_DETAILS
//#define OR_PRINT_ALGORITHM_AND_TIME_DETAILS_ALL

//#define DEMO_TO_CUSTOMER_HIDE_TRIANGLES_FROM_VIEW		//for 2DOD, this will stop data outside of the normalised triangle object data from being culled
#define OR_GENERATE_IMAGE_COMPARITOR_RESULTS_FNAME "imageComparitorResultsForNormalisedSnapshots.html"
#ifdef OR_GENERATE_IMAGE_COMPARITOR_RESULTS_HTML
	#ifndef OR_GENERATE_IMAGE_COMPARITOR_RESULTS_ALLOW_CONFIDENTIAL
		#define DEMO_TO_CUSTOMER_HIDE_TRIANGLES_FROM_VIEW
	/*
	#elif defined OR_USE_STAR_MAPS
		#define DEMO_TO_CUSTOMER_HIDE_TRIANGLES_FROM_VIEW
	*/
	#endif
#endif



	/**********
	OR ACCURACY/TUNING VARIABLES;
	***********/

#ifdef OR_USE_OLD_BUGGY_BUT_WORKS_CODE
	#define OR_IMAGE_COMPARITOR_MATCH_ERROR_THRESHOLD (0.3)
#else
	#ifdef OR_USE_STAR_MAPS
		#define OR_IMAGE_COMPARITOR_MATCH_ERROR_THRESHOLD (0.01)
	#else
		#define OR_IMAGE_COMPARITOR_MATCH_ERROR_THRESHOLD (0.2)	//default; 0.2
	#endif
#endif


	/**********
	OR OPTIMISATION VARIABLES2;
	***********/

//8. optimisation:  temp debug;
//#define TEMP_ALREADY_TRAINED
//#define DEBUG_ONLY_GENERATE_FEATURE_LISTS_NOTHING_ELSE

//9. optimisation: general;
#define OR_IMAGE_COMPARISON_USE_NEW_COMPARITOR	//test step 3
	#define OR_IMAGE_COMPARISON_DEBUG_NEW_COMPARITOR	//test step 3b
//#define OR_USE_OLD_BUGGY_BUT_WORKS_CODE		//test step 1	- works, but is now superceeded
#define OR_USE_HEITGER_OBJECT_FEATURE_CALCULATION_CODE			//by default, use OR_USE_HEITGER_OBJECT_FEATURE_CALCULATION_CODE
#ifdef COMPILE_TH_OR_METHOD_2DOD	//NB heitger ascii keypoint generation does not currently work with 3D data due to lack of depth checks
	
		//no longer use heither ascii keypoint generation - require subpixel accuracy
#endif

//10. optimisation: memory
#define OR_OPTIMISATION_DO_NOT_SAVE_REFERENCE_LIST_TO_FILE
#define OR_OPTIMISATION_DO_NOT_CREATE_UNNECESSARY_ARRAYS

//11. optimisation: hardware acceleration
#define USE_HARDWARE_ACCELERATION	//Default configuration is to use this!
#ifdef USE_HARDWARE_ACCELERATION
	#define RT_NO_TEXT_FEEDBACK
	#define USE_OPENGL		//must have freeglut installed
	#define USE_OPENGL_PREDEFINED_OD_MATRIX_OPERATIONS
	#define USE_OPENGL_PREDEFINED_OD_MATRIX_OPERATIONS_ADV

	//#define USE_OPENGL_OPTIMISED_TRIANGLE_RENDERING		//this is slower
#endif


//old versions;
//#define OR_METHOD_3DOD_USE_OLD_TESTED_BUT_BASIC_FEATURE_DETECTION	//not using this results in a problem, some features are off the object by 1 pixel;
	//#define DO_NOT_USE_CENTRE_PIXEL_REQ	//BAD


/* old;
//#define TEMP_TEST_HEITGER_FEATURE_THRESHOLD_IMAGE_SIZE_DEPENDENCE	//doesnt help
//#define TEMP_DEBUG_CONVERT
//#define TEMP_DEBUG_CONVERT2
//#define TEST_TIMES
//#define TEST_FEATURE_DETECTION_THRESHOLD_ON_A_NEW_OBJECT
#ifdef TEST_FEATURE_DETECTION_THRESHOLD_ON_A_NEW_OBJECT
#endif
*/



	/**********
	OR DEBUG VARIABLES 2;
	***********/

#define OR_DEBUG_DONT_WORRY_ABOUT_DISPLAYING_2D_NET_PROPERLY
#define OR_DEBUG_METHOD3DOD_POV
//#define OR_DEBUG_ONLY_WANT_TRAIN_VS_TEST_RGB_FILE_KNOWLEDGE
#ifdef OR_DEBUG_ONLY_WANT_TRAIN_VS_TEST_RGB_FILE_KNOWLEDGE
	#define OR_DEBUG_USE_ONLY_ONE_POLY
#endif
//#define OR_DEBUG_TRAIN_USING_FAR_INSUFFICIENT_NUMBER_EPOCHS
//#define OR_DEBUG_RT_LIGHTING_OR_SURFACE_COLOUR_REPEATABILITY
//#define OR_DEBUG_UNTHRESHOLDED_NN_FEED
//#define OR_DEBUG_RAYTRACE_INTERPOLATED_3D_MAP_WITH_ORIGINAL_VIEW
//#define OR_DEBUG_OLD_FEATURE_GENERATION_METHOD	//Old method is not working properly; need to translate corner coordinates into interpolated map coordiantes system








	/**********
	COMPILATION TYPE DEPENDENT VARIABLES;
	***********/


#ifdef COMPILE_CS
	#define USE_CS
	#define COMPILE_TH_OR_METHOD_2DOD_WITH_ANN
	#define CS_USE_RAINBOW_COLOURS_FOR_FILE_BOXES		//optional
	//#define CS_USE_RAINBOW_COLOURS_FOR_FUNCTION_BOXES	//optional
	#define CS_DISPLAY_INCLUDE_FILE_PARSING
	#define CS_CENTRE_BOXES
	#define CS_DO_NOT_DRAW_ALL_FUNCTION_CONNECTIONS_WHEN_TRACING_A_BOTTOM_LEVEL_FUNCTION_UPWARDS
	//#define CS_DO_NOT_DRAW_ALL_FUNCTION_BOXES_AND_TEXT_WHEN_TRACING_A_BOTTOM_LEVEL_FUNCTION_UPWARDS
	//#define CS_DO_NOT_DRAW_ALL_FILE_CONNECTIONS_WHEN_TRACING_A_BOTTOM_LEVEL_FUNCTION_UPWARDS
	//#define CS_DO_NOT_DRAW_ALL_FILE_BOXES_AND_TEXT_WHEN_TRACING_A_BOTTOM_LEVEL_FUNCTION_UPWARDS
#endif


#ifdef COMPILE_TH_OR_METHOD_2DOD_WITH_ANN
	#define USE_ANN
	#define COMPILE_TH_OR_METHOD_2DOD
#endif

#ifdef COMPILE_TH_OR_METHOD_3DOD_WITH_ANN
	#define USE_ANN
	#define COMPILE_TH_OR_METHOD_3DOD
#endif




#ifdef COMPILE_TH_OR_METHOD_2DOD
	#define USE_OR_METHOD_2DOD
	#define USE_OR
	#define USE_RT
	#ifdef USE_ANN
		#define OR_USE_OR_NEURAL_NETWORK_COMPARITOR
	#else
		#define OR_USE_RBB_IMAGE_COMPARITOR_INSTEAD_OF_NN
	#endif

	//#define OR_METHOD2DOD_GENERATE_IMAGE_DATA
	#define OR_METHOD2DOD_USE_PREEXISTING_IMAGE_DATA
	#ifdef OR_METHOD2DOD_USE_PREEXISTING_IMAGE_DATA

		#define OR_METHOD2DOD_TEST_ACROSS_MULTIPLE_VIEWS			//this scenario has not yet been coded properly for 3D method; need to work out appropriate camera coordinates for all view angles
		#define OR_USE_SINGLE_OBJECT


		//#define OR_USE_STAR_MAPS
		#define OR_USE_AMSTERDAM_TEST_IMAGE_DATA
		//#define OR_USE_GOOGLE_MAPS

		#ifdef OR_USE_STAR_MAPS
			#define OR_USE_SINGLE_TRAIN_STAR_MAP
			//#define OR_DO_NOT_IGNORE_BACKGROUND_COLOUR
			#define OR_METHOD_2DOD_ASSUME_NO_3D_ROTATION
		#endif

		#ifdef OR_USE_GOOGLE_MAPS
			//#define OR_USE_CROPPED_GOOGLE_MAPS_FOR_BEST_CASE_SCENARIO_TEST	//temp

			//currently testing google map OR heitger feature calculation postprocessor code
			#define OR_USE_HEITGER_OBJECT_FEATURE_CALCULATION_POST_PROCESSOR 	//a post processor is required, but it has not yet been implemented - the post processor will eliminate all features which are not distinct features, thereby reducing total number of features
		#endif


	#elif defined OR_METHOD2DOD_GENERATE_IMAGE_DATA
		#define OR_DEBUG_ONLY_USE_SMALL_NUM_POLYGONS
		#define OR_USE_SINGLE_OBJECT
		//#define OR_METHOD2DOD_TEST_ACROSS_MULTIPLE_VIEWS		//this scenario has not yet been coded properly for 3D method; need to work out appropriate camera coordinates for all view angles
	#endif

	/*
	#ifdef OR_USE_STAR_MAPS
		#define OR_METHOD_MINIMUM_AREA_OF_NORMALISATION_POLYGON (400)	//2500
	#else
		
		#define OR_METHOD_SIDE_LENGTH_FOR_MINIMUM_SIDE_LENGTH_RATIO_OF_EQUILATERAL_T (20)
		#define OR_METHOD_MINIMUM_SIDE_LENGTH_RATIO_OF_EQUILATERAL_T (5) //ie the minimum allowed is a 4x1 sided triangle

	#endif
	*/
	
#endif


#ifdef OR_USE_HEITGER_OBJECT_FEATURE_CALCULATION_CODE
	//#assert IMAGEMAGIK_INSTALLED
#endif
#ifdef OR_METHOD2DOD_USE_PREEXISTING_IMAGE_DATA
	//#assert IMAGEMAGIK_INSTALLED
#endif
#ifdef OR_GENERATE_IMAGE_COMPARITOR_RESULTS_HTML
	//#assert IMAGEMAGIK_INSTALLED
#endif


#ifdef COMPILE_TH_OR_METHOD_3DOD
	#define USE_OR_METHOD_3DOD
	#define USE_OR
	#define USE_RT
	#ifdef USE_ANN
		#define OR_USE_OR_NEURAL_NETWORK_COMPARITOR
	#else
		#define OR_USE_RBB_IMAGE_COMPARITOR_INSTEAD_OF_NN
	#endif
	//#define OR_DEBUG_RAYTRACE_INTERPOLATED_3D_MAP_WITH_ORIGINAL_VIEW
	#define OR_METHOD3DOD_GENERATE_IMAGE_DATA
	#define OR_USE_SINGLE_OBJECT
	#define OR_DEBUG_ONLY_USE_SMALL_NUM_POLYGONS
	#define OR_METHOD_MINIMUM_AREA_OF_NORMALISATION_POLYGON (0.1)	//this needs to be made dynamic in the future! [based upon distance object is away from pov]

	//#define OR_METHOD2DOD_USE_VIEWPORT_DIMENSION_BASED_ON_TRIANGLE_SIZE	//normalised snapshot size is proportional to ot size - this is not the standard 3DOD method

	//untested;
	//#define OR_METHOD3DOD_TEST_ACROSS_MULTIPLE_VIEWS
	//#define OR_METHOD_3DOD_ASSUME_TRAIN_AND_TEST_WITH_MULTI_VIEWPOINT_ADVANCED_VERSION	//required if more than 1 train viewpoint
	//#define OR_METHOD_3DOD_ASSUME_TRAIN_AND_TEST_WITH_SINGLE_VIEWPOINT	//dont need to use this right now, but worth testing anyway


#endif

#ifdef OR_USE_OR_NEURAL_NETWORK_COMPARITOR
	#define OR_TEST_ORI_AND_POS_NOISE_DURING_TRANING_AND_TESTING
	#define OR_DEBUG_DO_NOT_USE_ORI_NOISE
	#define OR_MANY_NUMBER_OF_OUTPUT_FILES
#endif

#ifdef COMPILE_ANN
	#define USE_ANN
	#define TH_USE_POVRAY_FOR_NEURAL_NETWORK_VEC_GRAPHICS
#endif

#ifdef COMPILE_ANN_WITH_RT
	#define USE_ANN
	#define TH_USE_RT_FOR_NEURAL_NETWORK_VEC_GRAPHICS
	#define USE_RT
#endif

#ifdef COMPILE_RT
	#define USE_RT
#endif

#ifdef COMPILE_TH_RT
	#define USE_RT
#endif



#ifdef COMPILE_LRRC
	#define USE_LRRC
#endif

#ifdef COMPILE_LRRC_WITH_ANN
	#define USE_LRRC
	#define USE_ANN
#endif

#ifdef COMPILE_TH_ANN_WITH_LRRC
	#define USE_LRRC
	#define USE_ANN
	#define TH_USE_ANN_TEST_HARNESS
	#define USE_RT
#endif




#ifdef TH_USE_ANN_TEST_HARNESS

	//#define DEMONSTRATION_SCENARIO1		//combat/properties experience testing
	#define DEMONSTRATION_SCENARIO2			//object experience testing
	//#define DEMONSTRATION_SCENARIO3		//contains functions not yet tested


		//Primary variables been tested by TH;
	#ifdef DEMONSTRATION_SCENARIO1
		//#define TH_GAME_USE_OBJECT_RECOGNITION_EXPERIENCES
			//#define TH_USE_POVRAY_FOR_OBJECT_RECOGNITION_VEC_GRAPHICS
			//#define TH_USE_RT_FOR_OBJECT_RECOGNITION_VEC_GRAPHICS
		#define TH_LRRC_GAME_USE_CC_EXPERIENCES
		#define TH_LRRC_GAME_USE_LD_EXPERIENCES
		//#define TH_LRRC_GAME_USE_GLOBAL_EXPERIENCES
		#define TH_GAME_RAY_TRACE_NEURAL_NETWORK_VEC_GRAPHICS
			//#define TH_USE_POVRAY_FOR_NEURAL_NETWORK_VEC_GRAPHICS
			#define TH_USE_RT_FOR_NEURAL_NETWORK_VEC_GRAPHICS

	#elif defined DEMONSTRATION_SCENARIO2
		#define TH_GAME_USE_OBJECT_RECOGNITION_EXPERIENCES
			#define TH_USE_POVRAY_FOR_OBJECT_RECOGNITION_VEC_GRAPHICS
			//#define TH_USE_RT_FOR_OBJECT_RECOGNITION_VEC_GRAPHICS
		//#define TH_LRRC_GAME_USE_CC_EXPERIENCES
		//#define TH_LRRC_GAME_USE_LD_EXPERIENCES
		//#define TH_LRRC_GAME_USE_GLOBAL_EXPERIENCES
		#define TH_GAME_RAY_TRACE_NEURAL_NETWORK_VEC_GRAPHICS
			//#define TH_USE_POVRAY_FOR_NEURAL_NETWORK_VEC_GRAPHICS
			#define TH_USE_RT_FOR_NEURAL_NETWORK_VEC_GRAPHICS

	#elif defined DEMONSTRATION_SCENARIO3
		#define TH_GAME_USE_OBJECT_RECOGNITION_EXPERIENCES
			//#define TH_USE_POVRAY_FOR_OBJECT_RECOGNITION_VEC_GRAPHICS
			#define TH_USE_RT_FOR_OBJECT_RECOGNITION_VEC_GRAPHICS
		//#define TH_LRRC_GAME_USE_CC_EXPERIENCES
		//#define TH_LRRC_GAME_USE_LD_EXPERIENCES
		//#define TH_LRRC_GAME_USE_GLOBAL_EXPERIENCES
		#define TH_GAME_RAY_TRACE_NEURAL_NETWORK_VEC_GRAPHICS
			#define TH_USE_POVRAY_FOR_NEURAL_NETWORK_VEC_GRAPHICS - this should be enabled!@
			//#define TH_USE_RT_FOR_NEURAL_NETWORK_VEC_GRAPHICS

	#else
		//#define TH_GAME_USE_OBJECT_RECOGNITION_EXPERIENCES
			//#define TH_USE_POVRAY_FOR_OBJECT_RECOGNITION_VEC_GRAPHICS
			//#define TH_USE_RT_FOR_OBJECT_RECOGNITION_VEC_GRAPHICS
		#define TH_LRRC_GAME_USE_CC_EXPERIENCES
		#define TH_LRRC_GAME_USE_LD_EXPERIENCES
		//#define TH_LRRC_GAME_USE_GLOBAL_EXPERIENCES
		//#define TH_GAME_RAY_TRACE_NEURAL_NETWORK_VEC_GRAPHICS
			//#define TH_USE_POVRAY_FOR_NEURAL_NETWORK_VEC_GRAPHICS
			//#define TH_USE_RT_FOR_NEURAL_NETWORK_VEC_GRAPHICS

	#endif



	//#define TEST_LAYER_DIVERGENCE_TYPE_LINEAR_DIVERGING_THEN_CONVERGING	//Default=not yet decided upon



		//conditions required for TH
	#ifndef TH_LRRC_GAME_USE_GLOBAL_EXPERIENCES
		#define DEBUG_DO_NOT_USE_GLOBAL_EXPERIENCES
	#endif
	#define DEBUG_DO_NOT_PERFORM_KILL_RATIO_CHECKS_BEFORE_ADDING_UNIT_EXPERIENCES_TO_NN
	#define DEBUG_DO_NOT_DISPLAY_COMBAT_RESULTS
	#define DEBUG_STORE_EXPERIENCES_IN_PLAYER_NOT_UNITS
	#define DEBUG_HAVNT_YET_STARTED_TESTING_UNITGROUPS


		//variables no longer changed
	#ifndef DEMONSTRATION_SCENARIO1
		#define TH_LRRC_GAME_EMULATE_GAME_AI_REQUIREMENTS	//!! //Default=defined	[will use randomised distances if defined]
	#endif
		#define DEBUG_DO_NOT_USE_RANDOM_DICE_DURING_COMBAT


		//Old tests no longer changed
	//#define ANN_SEPARATE_CC_FROM_LD_NETWORK		//Default=not defined

	/*TEMP DEBUG PERIOD SHOULD NEVER BE REQUIRED AGAIN;
	//#define DEGDISREGARD_HIGH_ERROR_EXPERIENCES
		//TEMP!
		//#define TEMP_OUTPUT_HIGH_ERROR_EXPERIENCES
	*/

	/*LOW RAM ALGORITHM DOES NOT WORK YET;
	//#define TH_LRRC_LOW_RAM_AVAILABLE	//default= not defined
	#ifdef TH_LRRC_LOW_RAM_AVAILABLE
		#define TH_LRRC_LOW_RAM_NUM_CONSEC_TRAINING_EPOCHS_PER_EXPERIENCE 50
	#endif
	*/

#endif

	//Additional Debug Scenarios;

//#define DEBUG_DO_NOT_USE_ADVANCED_LEARNING_BASED_UPON_DISTANCES
#ifdef DEBUG_DO_NOT_USE_ADVANCED_LEARNING_BASED_UPON_DISTANCES
	#define DEBUG_DO_NOT_USE_GLOBAL_EXPERIENCES
#endif

#define ANN_USE_MIN_NUM_EPOCHS_BEFORE_DECLARE_OVERFITTING_DATA
#ifdef ANN_USE_MIN_NUM_EPOCHS_BEFORE_DECLARE_OVERFITTING_DATA
	#define ANN_MIN_NUM_EPOCHS_BEFORE_DECLARE_OVERFITTING_DATA (5)	//10
#endif

#ifdef TH_GAME_USE_OBJECT_RECOGNITION_EXPERIENCES
	#define DEBUG_ANN_OBJECT_RECOGNITION_EXPERIENCES_OUTPUT_ALL_MAPS
	#define USE_HI_RES_PRIMITIVE_REPLACEMENT_DURING_RENDERING
	//#define ANN_OBJECT_RECOGNITION_EXPERIENCES_RANDOMISE_LIGHTING_CONDITIONS	//not available yet

	//#define GAME_OBJECT_RECOGNITION_EXPERIENCE_FEED_RGB_MAP
	//#define GAME_OBJECT_RECOGNITION_EXPERIENCE_FEED_LUMINOSITY_MAP
	//#define GAME_OBJECT_RECOGNITION_EXPERIENCE_FEED_LUMINOSITY_BOOLEAN_MAP
	//#define GAME_OBJECT_RECOGNITION_EXPERIENCE_LUMINOSITY_CONTRAST_MAP
	#define GAME_OBJECT_RECOGNITION_EXPERIENCE_LUMINOSITY_CONTRAST_BOOLEAN_MAP	//DEFAULT
#endif


#define DEBUG_ANN_VERIFY_REAL_LIFE_ACCURACY_DIAGNOSTICS_CUTOFF
#ifdef DEBUG_ANN_VERIFY_REAL_LIFE_ACCURACY_DIAGNOSTICS_CUTOFF
	#define DEBUG_ANN_PERFORM_ALTERNATE_ERROR_TOTAL
#endif

//#define LRRC_GAME_DO_NOT_USE_FOLDS_DURING_1DNN_TRAINING





//Program Type variables



//Debug variables
#define DEBUG_PERFORM_ONLY_1_GAME
#define DEBUG_IN_TEXTPAD
//#define DEBUG_NO_UNITGROUP_AI



//#define DEBUG_DO_NOT_PERFORM_KILL_RATIO_CHECKS_BEFORE_ADDING_UNIT_EXPERIENCES_TO_NN
//#define DEBUG_NO_CREATIVITY_ALWAYS_PERFORM_COMBAT_WITH_LOWEST_ERROR_UNIT
//#define DEBUG_NO_CREATIVITY_ALWAYS_PERFORM_MOVEMENT_INSTEAD_OF_STAND_FOR_LDCOMBAT
//#define DEBUG_SPRITES
//#define DEBUG_AI_MOVE_UNIT
//#define DEBUG_ADD_INDIVIDUAL_SPRITES
//#define OR_DEBUG_METHOD_2DOD


//Test variables (need optimisation)
#define MAX_ANN_BACK_PROPAGATION_ERROR (1000.0)


#define CHAR_NEWLINE '\n'
#define CHAR_SPACE ' '




#endif
